/*
 * Copyright (C) 1991, 1993 ToM Project
 * All rights reserved.
 *
 * Convert an ordinary file to an assembler file.
 *
 *	Usage: f2o file_name [object_file_name [symbol_name [size_name]]]
 *
 * Created by Tatsuya Hagino
 */
#include <stdio.h>

#define USAGE "Usage: %s file_name [asm_file_name [symbol_name [size_name]]]\n"

main(argc, argv)
int argc;
char **argv;
{
  char *program, *filename, *asmname, *symbolname, *sizename;
  FILE *file, *asm;
  register int i, c, ch;
  register long x;
  long size;

  program = argv[0];
  argc--;
  argv++;
  filename = asmname = symbolname = sizename = NULL;
  if (argc > 0) {
    filename = argv[0];
    argc--;
    argv++;
  }
  else {
    fprintf(stderr, USAGE, program);
    exit(1);
  }
  if (argc > 0) {
    asmname = argv[0];
    argc--;
    argv++;
  }
  else {
    asmname = (char *)malloc(strlen(filename)+3);
    strcpy(asmname, filename);
    strcat(asmname, ".s");
  }
  if (argc > 0) {
    symbolname = argv[0];
    argc--;
    argv++;
  }
  else {
    symbolname = (char *)malloc(strlen(filename)+2);
    strcpy(symbolname, "_");
    strcat(symbolname, filename);
  }

  if (argc > 0) {
    sizename = argv[0];
    argc--;
    argv++;
  }
  else {
    sizename = (char *)malloc(strlen(symbolname)+strlen("_size")+1);
    strcpy(sizename, symbolname);
    strcat(sizename, "_size");
  }

  if (!(file = fopen(filename, "r"))) {
    fprintf(stderr, "%s: cannot read %s\n", program, filename);
    exit(1);
  }
  if (!(asm = fopen(asmname, "w"))) {
    fprintf(stderr, "%s: cannot write %s\n", program, asmname);
    exit(1);
  }

  fseek(file, 0L, 2);	/* go to the end of file */
  size = ftell(file);
  fseek(file, 0L, 0);	/* rewind */

  fprintf(asm, "\t.verstamp\t2 11\n");
  fprintf(asm, "\t.globl\t%s\n", sizename);
  fprintf(asm, "\t.sdata\n");
  fprintf(asm, "\t.align\t2\n");
  fprintf(asm, "\t.align\t0\n");
  fprintf(asm, "%s:\n", sizename);
  fprintf(asm, "\t.word\t%d : 1\n", size);
  fprintf(asm, "\t.globl\t%s\n", symbolname);
  fprintf(asm, "\t.data\n");
  fprintf(asm, "\t.align\t2\n");
  fprintf(asm, "\t.align\t0\n");
  fprintf(asm, "%s:\n", symbolname);

  for (i = 0; i < size; i += 4) {
    x = ((getc(file)&0xff)<<24)+((getc(file)&0xff)<<16)+((getc(file)&0xff)<<8)+(getc(file)&0xff);
    fprintf(asm, "\t.word\t%u : 1\n", x);
  }
  fclose(file);
  fclose(asm);

  exit(0);
}
